from aiohttp import ClientSession

from .types import GoogleTranslateResult

import requests


class GoogleTranslator:
    def __init__(self) -> None:
        pass

    def translate(self, text: str, dest: str = "en") -> "GoogleTranslateResult":
        response = requests.get(
            f"https://clients5.google.com/translate_a/t?client=at&sl=auto&tl={dest}&q={text}"
        )
        try:
            result = response.json()
        except Exception as e:
            raise BaseException(str(e))

        return GoogleTranslateResult.parse(result[0], dest)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return self


class AsyncGoogleTranslator:
    def __init__(self) -> None:
        pass

    async def translate(self, text: str, dest: str = "en") -> "GoogleTranslateResult":
        async with ClientSession() as session:
            async with session.get(
                f"https://clients5.google.com/translate_a/t?client=at&sl=auto&tl={dest}&q={text}"
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    raise BaseException(str(e))

                return GoogleTranslateResult.parse(result[0], dest)

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        return self



from aiohttp import ClientSession

from .types import AjaxTranslateResult
from .exceptions import ApiException

import requests


class AjaxTranslator:
    def __init__(self) -> None:
        pass

    def translate(self, text: str, dest: str = "en") -> "AjaxTranslateResult":
        org = self.detect(text)
        response = requests.post(
            "https://www.translate.com/translator/ajax_translate",
            data={
                "text_to_translate": text,
                "translated_lang": dest,
                "source_lang": org,
                "use_cache_only": "false",
            },
        )
        try:
            result = response.json()
        except Exception as e:
            return BaseException(str(e))

        if not result["result"] == "success":
            raise ApiException(result["message"])

        return AjaxTranslateResult.parse(result, org)

    def detect(self, text: str) -> str:
        response = requests.post(
            "https://www.translate.com/translator/ajax_lang_auto_detect",
            data={"text_to_translate": text},
        )
        try:
            result = response.json()
        except Exception as e:
            return BaseException(str(e))
        if not result["result"] == "success":
            raise ApiException(result["message"])

        return result["language"]

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return self


class AsyncAjaxTranslator:
    def __init__(self) -> None:
        pass

    async def translate(self, text: str, dest: str = "en") -> "AjaxTranslateResult":
        org = await self.detect(text)
        async with ClientSession() as session:
            async with session.post(
                "https://www.translate.com/translator/ajax_translate",
                data={
                    "text_to_translate": text,
                    "translated_lang": dest,
                    "source_lang": org,
                    "use_cache_only": "false",
                },
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    return BaseException(str(e))
                if not result["result"] == "success":
                    raise ApiException(result["message"])

            return AjaxTranslateResult.parse(result, org)

    async def detect(self, text: str) -> str:
        async with ClientSession() as session:
            async with session.post(
                "https://www.translate.com/translator/ajax_lang_auto_detect",
                data={"text_to_translate": text},
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    return BaseException(str(e))
                if not result["result"] == "success":
                    raise ApiException(result["message"])

                return result["language"]

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        return self



import requests
import aiohttp
import json
import os

from aiofiles import open as AsyncOpen

from .exceptions import ApiException


class OCR:
    def __init__(self) -> None:
        pass

    def from_image(self, path: str, language: str = "eng") -> str:
        with open(path, "rb") as f:
            response = requests.post(
                "https://api8.ocr.space/parse/image",
                data={
                    "language": language,
                    "isOverlayRequired": True,
                    "OCREngine": 1,
                    "detectCheckbox": False,
                    "IsCreateSearchablePDF": False,
                    "isSearchablePdfHideTextLayer": True,
                    "FileType": ".AUTO",
                },
                headers={
                    "Apikey": "donotstealthiskey_ip1",
                },
                files={"file": f},
            )
            try:
                result = response.json()
            except Exception as e:
                raise BaseException(str(e))

            if isinstance(result, str):
                raise ApiException(result)
            if not result.get("ParsedResults"):
                raise ApiException(json.dumps(result, indent=4, ensure_ascii=False))

            return result["ParsedResults"][0]["ParsedText"]


class AsyncOCR:
    def __init__(self) -> None:
        pass

    async def from_image(self, path: str, language: str = "eng") -> str:
        async with AsyncOpen(path, "rb") as f:
            async with aiohttp.ClientSession(
                headers={"Apikey": "donotstealthiskey_ip1"}
            ) as session:
                payload = {
                    "language": language,
                    "isOverlayRequired": True,
                    "OCREngine": 1,
                    "detectCheckbox": False,
                    "IsCreateSearchablePDF": False,
                    "isSearchablePdfHideTextLayer": True,
                    "FileType": ".AUTO",
                }
                data = aiohttp.formdata.FormData(quote_fields=False)
                for k, v in payload.items():
                    data.add_field(k, str(v))
                data.add_field("file", await f.read(), filename=os.path.basename(path))
                async with session.post(
                    "https://api8.ocr.space/parse/image", data=data
                ) as response:
                    try:
                        result = await response.json()
                    except Exception as e:
                        raise BaseException(str(e))

                    if isinstance(result, str):
                        raise ApiException(result)
                    if not result.get("ParsedResults"):
                        raise ApiException(
                            json.dumps(result, indent=4, ensure_ascii=False)
                        )

                    return result["ParsedResults"][0]["ParsedText"]



from aiohttp import ClientSession

import requests


class TdictTranslator:
    def __init__(self) -> None:
        pass

    def translate(self, text: str, dest: str = "en") -> str:
        response = requests.get(
            f"https://t3.translatedict.com/1.php?p1=auto&p2={dest}&p3={text}"
        )
        return response.text

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return self


class AsyncTdictTranslator:
    def __init__(self) -> None:
        pass

    async def translate(self, text: str, dest: str = "en") -> str:
        async with ClientSession() as session:
            async with session.get(
                f"https://t3.translatedict.com/1.php?p1=auto&p2={dest}&p3={text}"
            ) as response:
                return await response.text("utf-8")

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        return self



from aiohttp import ClientSession

from .types import HozoryTranslateResult
from .exceptions import ApiException

from json import dumps

import requests


class HozoryTranslator:
    def __init__(self) -> None:
        pass

    def translate(self, text: str, dest: str = "en") -> "HozoryTranslateResult":
        response = requests.get(
            f"https://hozory.com/translate/?target={dest}&text={text}"
        )
        try:
            result = response.json()
        except Exception as e:
            raise BaseException(str(e))

        if not result["status"] == "ok":
            raise ApiException(dumps(result["result"], indent=2, ensure_ascii=False))

        return HozoryTranslateResult.parse(result["result"], dest)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return self


class AsyncHozoryTranslator:
    def __init__(self) -> None:
        pass

    async def translate(self, text: str, dest: str = "en") -> "HozoryTranslateResult":
        async with ClientSession() as session:
            async with session.get(
                f"https://hozory.com/translate/?target={dest}&text={text}"
            ) as response:
                try:
                    result = await response.json()
                except Exception as e:
                    raise BaseException(str(e))

                if not result["status"] == "ok":
                    raise ApiException(
                        dumps(result["result"], indent=2, ensure_ascii=False)
                    )

                return HozoryTranslateResult.parse(result["result"], dest)

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        return self



from . import ajax, google, hozory, tdict, ocr

__all__ = ["AsyncEngine", "Engine"]


class Engine:
    def __init__(self) -> None:
        pass

    @property
    def ajax(self) -> "ajax.AjaxTranslator":
        return ajax.AjaxTranslator()

    @property
    def google(self) -> "google.GoogleTranslator":
        return google.GoogleTranslator()

    @property
    def hozory(self) -> "hozory.HozoryTranslator":
        return hozory.HozoryTranslator()

    @property
    def tdict(self) -> "tdict.TdictTranslator":
        return tdict.TdictTranslator()

    @property
    def ocr(self) -> "ocr.OCR":
        return ocr.OCR()


class AsyncEngine:
    def __init__(self) -> None:
        pass

    @property
    def ajax(self) -> "ajax.AsyncAjaxTranslator":
        return ajax.AsyncAjaxTranslator()

    @property
    def google(self) -> "google.AsyncGoogleTranslator":
        return google.AsyncGoogleTranslator()

    @property
    def hozory(self) -> "hozory.AsyncHozoryTranslator":
        return hozory.AsyncHozoryTranslator()

    @property
    def tdict(self) -> "tdict.AsyncTdictTranslator":
        return tdict.AsyncTdictTranslator()

    @property
    def ocr(self) -> "ocr.AsyncOCR":
        return ocr.AsyncOCR()



from dataclasses import dataclass
from json import dumps


class Base:
    @staticmethod
    def default(obj: "Base"):
        return {
            **{
                attr: (getattr(obj, attr))
                for attr in filter(lambda x: not x.startswith("_"), obj.__dict__)
                if getattr(obj, attr) is not None
            },
        }

    def __str__(self) -> str:
        return dumps(self, indent=4, default=Base.default, ensure_ascii=False)


@dataclass
class GoogleTranslateResult(Base):
    translated_text: str
    original_language: str
    dest_language: str

    @staticmethod
    def parse(d: list, dest: str) -> "GoogleTranslateResult":
        return GoogleTranslateResult(
            translated_text=" ".join([i for i in d[:-1]]),
            original_language=d[-1],
            dest_language=dest,
        )


@dataclass
class AjaxTranslateResult(Base):
    original_text: str
    translated_text: str
    translation_language: str
    original_language: str

    @staticmethod
    def parse(d: dict, org: str) -> "AjaxTranslateResult":
        return AjaxTranslateResult(
            original_text=d["original_text"],
            translated_text=d["translated_text"],
            translation_language=d["human_translation_details"]["translation_language"],
            original_language=org,
        )


@dataclass
class HozoryTranslateResult(Base):
    translated_text: str
    translation_language: str
    voice_link: str

    @staticmethod
    def parse(d: dict, dest: str) -> "HozoryTranslateResult":
        return HozoryTranslateResult(
            translated_text=d["translate"],
            translation_language=dest,
            voice_link=d["Voice_link"],
        )
